import numpy as np
import pandas as pd 

def Pandas_Missing_Data( dataf , Percent_Missing = -1 ):
  mc = dataf.isnull().sum(axis = 0)
  pm = dataf.isnull().sum() * 100 / len(dataf)
  missing_value_df = pd.DataFrame({'column_name': dataf.columns,
                                 'missing_count' : mc,
                                 'percent_missing': pm})
  return missing_value_df[missing_value_df['percent_missing'] > Percent_Missing]


def Constant_In_Data(dataf , n_unique = 1):
  uniquecols = dataf.apply(pd.Series.nunique)

  if n_unique == 1:
    rtn = uniquecols[uniquecols == 1].index.tolist()
  else :
    rtn = uniquecols[uniquecols <=  n_unique ].index.tolist()
  return rtn


def ID_in_Data(dataf ):
  ML = len(dataf)
  uniquecols = dataf.apply(pd.Series.nunique)
  return uniquecols[uniquecols == ML].index.tolist()



import numpy as np
import pandas as pd 
import statsmodels.api as sm
import matplotlib.pyplot as plt


class SPPSF_Polyniomial_Time_Model :


  def fit( self ,SPPSF_ , Time_ , return_model =False ):
    #Creates df for SPPSF and Months
    modelData = pd.DataFrame({
                              'SPPSF' : SPPSF_ , 
                              'const' : 1, 
                              'Months' : Time_})

    bestTimeModel = None
    for degree in range(1, len(modelData['Months'].unique())):
      if degree > 1:
          #modelData['Months' + str(degree)] = modelData['Months'] ** degree
          modelData['Months' + str(degree)] = modelData['Months'].pow(degree).copy()
          modelData = modelData.copy()
      m = sm.OLS(modelData['SPPSF'], modelData.drop('SPPSF', axis = 1)).fit()

      if(bestTimeModel == None or m.bic < bestTimeModel.bic):
          bestTimeModel = m

    self.Time_Model = bestTimeModel

    modelDataR = pd.DataFrame(bestTimeModel.model.data.exog
                              , columns = bestTimeModel.model.data.param_names
                              , index = bestTimeModel.model.data.row_labels)

    predData = pd.DataFrame({
                               'const' : 1
                               , 'Months' : range(0, modelData['Months'].max() + 1)} , index = range(0, modelData['Months'].max() + 1))

    if bestTimeModel.df_model > 1:
      for x in range(2, int(bestTimeModel.df_model) + 1):
        predData['Months' + str(x)] = predData['Months'] ** x
    self.pred_data = predData

    if return_model ==True:
      return bestTimeModel
  
  def Display_Time_Trend(self): 
    plt.plot(-self.pred_data['Months'], self.Time_Model.predict(self.pred_data), '-', color = 'red', linewidth = 3)
    plt.axhline(self.Time_Model.predict(self.pred_data).loc[0], color='k', linestyle='--')
    plt.show()

  def Adjustment_Rate_Return(self, as_pandas =False):
    if as_pandas == True :
      predDataResults = self.pred_data.copy()
      predDataResults['Model_Prediction'] = self.Time_Model.predict(self.pred_data)
      predDataResults['AdjustMent_Rate'] = self.Time_Model.predict(self.pred_data).loc[0] / self.Time_Model.predict(self.pred_data)
      return predDataResults[['Months','AdjustMent_Rate']]

    else:
      rateTable = self.Time_Model.predict(self.pred_data).loc[0] / self.Time_Model.predict(self.pred_data)
      rateTable.name = 'AdjRate'
      return rateTable
    
  def trend_summary(self):
    return self.Time_Model.summary()



import numpy as np
import pandas as pd 
import statsmodels.api as sm


def weighted_Mean_Sale_Ratio ( y, x ):
  return np.mean(x) / np.mean(y)

def COD( y, x):
  ratio = x / y
  med =  np.median(ratio)
  dev = np.sum(np.abs(ratio - med))
  avgdev=dev / len(ratio)
  cod = 100 * (avgdev / med)
  return cod


def PRD(y, x):
  ratio = x / y
  mnratio = np.mean(ratio)
  mnx = np.mean(x)
  mny = np.mean(y)
  prd = mnratio / (mnx /mny)
  return prd

  
def PRB (y , x ): 
  rtn = None  
  if len(x) <= 2:
    rtn = None
  else :
    ratio = x / y
    med =  np.median(ratio)
    avmed = x / med
    value = .5 * y + .5 * avmed
    ind = np.log(value) / np.log(2)
    dep = (ratio -med) / med
    ind2 = sm.add_constant(ind)
    reg = sm.OLS(dep, ind2).fit()
    if reg.pvalues[1]  < .1 :
      rtn =  reg.params[1]
    else :
      rtn = 0 
  return rtn
